@@ -92,9 +92,15 @@ struct _HDChangeBackgroundDialogPrivate
 
   guint         current_view;
 
+  guint scroll_to_selected_id;
+
   HildonThumbnailFactory *thumbnail_factory;
 };
 
+static void request_thumbnail_for_uri (HDChangeBackgroundDialog *dialog,
+                                       const char               *uri,
+                                       GtkTreeIter              *iter);
+
 G_DEFINE_TYPE (HDChangeBackgroundDialog, hd_change_background_dialog, GTK_TYPE_DIALOG);
 
 static gchar *
@@ -362,22 +368,11 @@ hd_change_background_dialog_append_backgrounds (HDChangeBackgroundDialog  *dialo
         }
       else if (image_set)
         {
-          GtkTreePath *path;
-          GtkTreeRowReference *reference;
           gchar *uri = g_filename_to_uri (image[1], NULL, NULL);
 
-          path = gtk_tree_model_get_path (priv->model, &iter);
-          reference = gtk_tree_row_reference_new (priv->model, path);
-          gtk_tree_path_free (path);
-
-          hildon_thumbnail_factory_request_pixbuf (priv->thumbnail_factory,
-                                                   uri,
-                                                   80, 60,
-                                                   FALSE,
-                                                   NULL,
-                                                   image_set_thumbnail_callback,
-                                                   reference,
-                                                   (GDestroyNotify) gtk_tree_row_reference_free);
+          request_thumbnail_for_uri (dialog,
+                                     uri,
+                                     &iter);
 
           g_free (uri);
         }
@@ -416,6 +411,30 @@ cleanup:
   g_dir_close (dir);
 }
 
+static void
+request_thumbnail_for_uri (HDChangeBackgroundDialog *dialog,
+                           const char               *uri,
+                           GtkTreeIter              *iter)
+{
+  HDChangeBackgroundDialogPrivate *priv = dialog->priv;
+  GtkTreePath *path;
+  GtkTreeRowReference *reference;
+
+  path = gtk_tree_model_get_path (priv->model, iter);
+  reference = gtk_tree_row_reference_new (priv->model, path);
+
+  hildon_thumbnail_factory_request_pixbuf (priv->thumbnail_factory,
+                                           uri,
+                                           80, 60,
+                                           FALSE,
+                                           NULL,
+                                           image_set_thumbnail_callback,
+                                           reference,
+                                           (GDestroyNotify) gtk_tree_row_reference_free);
+
+  gtk_tree_path_free (path);
+}
+
 static void
 hd_change_background_dialog_constructed (GObject *object)
 {
@@ -496,9 +515,10 @@ hd_change_background_dialog_constructed (GObject *object)
   else
     {
       GtkTreeIter iter;
-      gchar *label;
+      gchar *label, *uri;
 
       label = g_filename_display_basename (current_background);
+      uri = g_filename_to_uri (current_background, NULL, NULL);
 
       gtk_list_store_insert_with_values (GTK_LIST_STORE (priv->model),
                                          &iter,
@@ -510,6 +530,11 @@ hd_change_background_dialog_constructed (GObject *object)
                                          COL_IMAGE_SET, FALSE,
                                          -1);
 
+      if (uri)
+        request_thumbnail_for_uri (dialog,
+                                   uri,
+                                   &iter);
+
       priv->custom_image = gtk_tree_model_get_path (priv->model, &iter);
 
       hildon_touch_selector_select_iter (HILDON_TOUCH_SELECTOR (priv->selector),
@@ -517,6 +542,7 @@ hd_change_background_dialog_constructed (GObject *object)
                                          &iter,
                                          TRUE);
       g_free (label);
+      g_free (uri);
     }
 }
 
@@ -531,6 +557,9 @@ hd_change_background_dialog_dispose (GObject *object)
   if (priv->thumbnail_factory)
     priv->thumbnail_factory = (g_object_unref (priv->thumbnail_factory), NULL);
 
+  if (priv->scroll_to_selected_id)
+    priv->scroll_to_selected_id = (g_source_remove (priv->scroll_to_selected_id), 0);
+
   G_OBJECT_CLASS (hd_change_background_dialog_parent_class)->dispose (object);
 }
 
@@ -581,6 +610,24 @@ get_image_label_from_uri (const gchar *uri)
   return imagename;
 }
 
+static gboolean
+scroll_to_selected (gpointer data)
+{
+  HDChangeBackgroundDialogPrivate *priv = HD_CHANGE_BACKGROUND_DIALOG (data)->priv;
+  GtkTreeIter iter;
+
+  priv->scroll_to_selected_id = 0;
+
+  gtk_tree_model_get_iter (priv->model, &iter, priv->custom_image);
+
+  hildon_touch_selector_select_iter (HILDON_TOUCH_SELECTOR (priv->selector),
+                                     0,
+                                     &iter,
+                                     TRUE);
+
+  return FALSE;
+}
+
 static void
 add_image_dialog_response (GtkDialog                *dialog,
                            gint                      response_id,
@@ -596,6 +643,8 @@ add_image_dialog_response (GtkDialog                *dialog,
 
       uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
 
+      g_debug ("%s. Selected file: %s.", __FUNCTION__, uri);
+
       if (!uri)
         {
           g_warning ("No image file selected.");
@@ -624,16 +673,17 @@ add_image_dialog_response (GtkDialog                *dialog,
                               COL_ORDER, -2, /* first entry */
                               -1);
 
-          hildon_touch_selector_select_iter (HILDON_TOUCH_SELECTOR (priv->selector),
-                                             0,
-                                             &iter,
-                                             TRUE);
+          request_thumbnail_for_uri (cb_dialog,
+                                     uri,
+                                     &iter);
+
+          priv->scroll_to_selected_id = gdk_threads_add_idle (scroll_to_selected,
+                                                              cb_dialog);
 
           g_free (uri);
           g_free (filename);
           g_free (label);
         }
-
     }
 
   gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -725,25 +775,11 @@ hd_change_background_dialog_response (GtkDialog *dialog,
 
           if (image_set)
             {
-              guint i;
-
-              hd_backgrounds_set_background (hd_backgrounds_get (),
-                                             priv->current_view,
-                                             image[priv->current_view + 1],
-                                             NULL,
-                                             NULL);
-
-              for (i = 0; i < 4; i++)
-                {
-                  if (i != priv->current_view)
-                    hd_backgrounds_set_background (hd_backgrounds_get (),
-                                                   i,
-                                                   image[i + 1],
-                                                   i == 3 ||
-                                                   (i == 2 && priv->current_view == 3) ?
-                                                   background_set_cb : NULL,
-                                                   dialog);
-                }
+              hd_backgrounds_set_image_set (hd_backgrounds_get (),
+                                            &image[1],
+                                            background_set_cb,
+                                            dialog,
+                                            NULL);
             }
           else
             {
@@ -751,7 +787,8 @@ hd_change_background_dialog_response (GtkDialog *dialog,
                                              priv->current_view,
                                              image[0],
                                              background_set_cb,
-                                             dialog);
+                                             dialog,
+                                             NULL);
             }
 
           /* free memory */
